
#include <stdio.h>
#include <string.h>
#include "mqtt_port.h"
#include "aiot_state_api.h"
#include "mqtt_impl.h"
#include "core_log.h"

#define MQTT_IMPL "MQTT"

static aiot_sysdep_portfile_t *sysdep = NULL;

static int32_t _core_mqtt_sysdep_return(int32_t sysdep_code, int32_t core_code)
{
    if (sysdep_code >= (STATE_PORT_BASE - 0x00FF) && sysdep_code < (STATE_PORT_BASE)) {
        return sysdep_code;
    } else {
        return core_code;
    }
}

static uint16_t _core_mqtt_packet_id(mqtt_client_t *mqtt_client)
{
    uint16_t packet_id = 0;

    sysdep->core_sysdep_mutex_lock(mqtt_client->data_mutex);
    if ((uint16_t)(mqtt_client->packet_id + 1) == 0) {
        mqtt_client->packet_id = 0;
    }
    packet_id = ++mqtt_client->packet_id;
    sysdep->core_sysdep_mutex_unlock(mqtt_client->data_mutex);

    return packet_id;
}

static void _core_mqtt_set_utf8_encoded_str(uint8_t *input, uint16_t input_len, uint8_t *output)
{
    uint32_t idx = 0, input_idx = 0;

    /* String Length MSB */
    output[idx++] = (uint8_t)((input_len >> 8) & 0x00FF);

    /* String Length LSB */
    output[idx++] = (uint8_t)((input_len) & 0x00FF);

    /* UTF-8 Encoded Character Data */
    for (input_idx = 0; input_idx < input_len; input_idx++) {
        output[idx++] = input[input_idx];
    }
}

static void _core_mqtt_remain_len_encode(uint32_t input, uint8_t *output, uint32_t *output_idx)
{
    uint8_t encoded_byte = 0, idx = 0;

    do {
        encoded_byte = input % 128;
        input /= 128;
        if (input > 0) {
            encoded_byte |= 128;
        }
        output[idx++] = encoded_byte;
    } while (input > 0);

    *output_idx += idx;
}

static int32_t _core_mqtt_conn_pkt(conn_para_t *params, uint8_t **pkt, uint32_t *pkt_len)
{
    uint32_t idx = 0, conn_paylaod_len = 0, conn_remainlen = 0, conn_pkt_len = 0;

    uint8_t *pos = NULL;
    const uint8_t conn_fixed_header = CORE_MQTT_CONN_PKT_TYPE;
    const uint8_t conn_protocol_name[] = {0x00, 0x04, 0x4D, 0x51, 0x54, 0x54};
    const uint8_t conn_protocol_level = 0x04;
    const uint8_t conn_connect_flag = 0xC0 | (params->clean_session << 1);

    /* Payload Length */
    conn_paylaod_len = (uint32_t)(strlen(params->clientid) + strlen(params->username)
                                  + strlen(params->password) + 3 * CORE_MQTT_UTF8_STR_EXTRA_LEN);

    /* Remain-Length Value */
    conn_remainlen = CORE_MQTT_CONN_REMAINLEN_FIXED_LEN + conn_paylaod_len;

    /* Total Packet Length */
    conn_pkt_len = CORE_MQTT_CONN_FIXED_HEADER_TOTAL_LEN + conn_paylaod_len;

    pos = sysdep->core_sysdep_malloc(conn_pkt_len, MQTT_IMPL);
    if (pos == NULL) {
        return STATE_SYS_DEPEND_MALLOC_FAILED;
    }
    memset(pos, 0, conn_pkt_len);

    /* Fixed Header */
    pos[idx++] = conn_fixed_header;

    /* Remain Length */
    _core_mqtt_remain_len_encode(conn_remainlen, pos + idx, &idx);

    /* Protocol Name */
    memcpy(pos + idx, conn_protocol_name, CORE_MQTT_CONN_PROTOCOL_NAME_LEN);
    idx += CORE_MQTT_CONN_PROTOCOL_NAME_LEN;

    /* Protocol Level */
    pos[idx++] = conn_protocol_level;

    /* Connect Flag */
    pos[idx++] = conn_connect_flag;

    /* Keep Alive MSB */
    pos[idx++] = (uint8_t)((params->keep_alive_s >> 8) & 0x00FF);

    /* Keep Alive LSB */
    pos[idx++] = (uint8_t)((params->keep_alive_s) & 0x00FF);

    /* Payload: clientid, username, password */
    _core_mqtt_set_utf8_encoded_str((uint8_t *)params->clientid, strlen(params->clientid), pos + idx);
    idx += CORE_MQTT_UTF8_STR_EXTRA_LEN + strlen(params->clientid);

    _core_mqtt_set_utf8_encoded_str((uint8_t *)params->username, strlen(params->username), pos + idx);
    idx += CORE_MQTT_UTF8_STR_EXTRA_LEN + strlen(params->username);

    _core_mqtt_set_utf8_encoded_str((uint8_t *)params->password, strlen(params->password), pos + idx);
    idx += CORE_MQTT_UTF8_STR_EXTRA_LEN + strlen(params->password);

    *pkt = pos;
    *pkt_len = idx;

    return STATE_SUCCESS;
}

static int32_t _core_mqtt_read(mqtt_client_t *mqtt_client, uint8_t *buffer, uint32_t len, uint32_t timeout_ms)
{
    int32_t res = STATE_SUCCESS;

    if (mqtt_client->network_handle != NULL) {
        res = sysdep->core_sysdep_network_recv(mqtt_client->network_handle, buffer, len, timeout_ms, NULL);
        if (res < STATE_SUCCESS) {
            res = _core_mqtt_sysdep_return(res, STATE_SYS_DEPEND_NWK_RECV_ERR);
        } else if (res != len) {
            res = STATE_SYS_DEPEND_NWK_READ_LESSDATA;
        }
    } else {
        res = STATE_SYS_DEPEND_NWK_CLOSED;
    }

    return res;
}

static int32_t _core_mqtt_write(mqtt_client_t *mqtt_client, uint8_t *buffer, uint32_t len, uint32_t timeout_ms)
{
    int32_t res = STATE_SUCCESS;

    if (mqtt_client->network_handle != NULL) {
        res = sysdep->core_sysdep_network_send(mqtt_client->network_handle, buffer, len, timeout_ms, NULL);
        if (res < STATE_SUCCESS) {
            res = _core_mqtt_sysdep_return(res, STATE_SYS_DEPEND_NWK_SEND_ERR);
        } else if (res != len) {
            res = STATE_SYS_DEPEND_NWK_WRITE_LESSDATA;
        }
    } else {
        res = STATE_SYS_DEPEND_NWK_CLOSED;
    }

    return res;
}

static int32_t _core_mqtt_read_remainlen(mqtt_client_t *mqtt_client, uint32_t *remainlen)
{
    int32_t res = 0;
    uint8_t ch = 0;
    uint32_t multiplier = 1;
    uint32_t mqtt_remainlen = 0;

    do {
        res = _core_mqtt_read(mqtt_client, &ch, 1, mqtt_client->recv_timeout_ms);
        if (res < STATE_SUCCESS) {
            return res;
        }
        mqtt_remainlen += (ch & 127) * multiplier;
        if (multiplier > 128 * 128 * 128) {
            return STATE_MQTT_MALFORMED_REMAINING_LEN;
        }
        multiplier *= 128;
    } while ((ch & 128) != 0);

    *remainlen = mqtt_remainlen;

    return STATE_SUCCESS;
}

static int32_t _core_mqtt_read_remainbytes(mqtt_client_t *mqtt_client, uint32_t remainlen, uint8_t **output)
{
    int32_t res = 0;
    uint8_t *remain = NULL;

    if (remainlen > 0) {
        remain = sysdep->core_sysdep_malloc(remainlen, MQTT_IMPL);
        if (remain == NULL) {
            return STATE_SYS_DEPEND_MALLOC_FAILED;
        }
        memset(remain, 0, remainlen);

        res = _core_mqtt_read(mqtt_client, remain, remainlen, mqtt_client->recv_timeout_ms);
        if (res < STATE_SUCCESS) {
            sysdep->core_sysdep_free(remain);
            if (res == STATE_SYS_DEPEND_NWK_READ_LESSDATA) {
                return STATE_MQTT_MALFORMED_REMAINING_BYTES;
            } else {
                return res;
            }
        }
    }
    *output = remain;

    return STATE_SUCCESS;
}

static int32_t _core_mqtt_connack_handle(mqtt_client_t *mqtt_client, uint8_t *connack, uint32_t remain_len)
{
    int32_t res = STATE_SUCCESS;

    if (connack[1] == CORE_MQTT_CONNACK_RCODE_ACCEPTED) {
        res = STATE_SUCCESS;
    } else if (connack[1] == CORE_MQTT_CONNACK_RCODE_UNACCEPTABLE_PROTOCOL_VERSION) {
        core_log(sysdep, STATE_MQTT_LOG_DISCONNECT, "MQTT invalid protocol version, disconeect\r\n");
        res = STATE_MQTT_CONNACK_RCODE_UNACCEPTABLE_PROTOCOL_VERSION;
    } else if (connack[1] == CORE_MQTT_CONNACK_RCODE_SERVER_UNAVAILABLE) {
        core_log(sysdep, STATE_MQTT_LOG_DISCONNECT, "MQTT server unavailable, disconnect\r\n");
        res = STATE_MQTT_CONNACK_RCODE_SERVER_UNAVAILABLE;
    } else if (connack[1] == CORE_MQTT_CONNACK_RCODE_BAD_USERNAME_PASSWORD) {
        core_log(sysdep, STATE_MQTT_LOG_DISCONNECT, "MQTT bad username or password, disconnect\r\n");
        res = STATE_MQTT_CONNACK_RCODE_BAD_USERNAME_PASSWORD;
    } else if (connack[1] == CORE_MQTT_CONNACK_RCODE_NOT_AUTHORIZED) {
        core_log(sysdep, STATE_MQTT_LOG_DISCONNECT, "MQTT authorize fail, disconnect\r\n");
        res = STATE_MQTT_CONNACK_RCODE_NOT_AUTHORIZED;
    } else {
        res = STATE_MQTT_CONNACK_RCODE_UNKNOWN;
    }
    return res;
}

static int32_t _core_mqtt_subunsub(mqtt_client_t *mqtt_client, char *topic, uint16_t topic_len, uint8_t qos,
                                   uint8_t pkt_type)
{
    int32_t res = 0;
    uint16_t packet_id = 0;
    uint8_t *pkt = NULL;
    uint32_t idx = 0, pkt_len = 0, remainlen = 0;

    remainlen = CORE_MQTT_PACKETID_LEN + CORE_MQTT_UTF8_STR_EXTRA_LEN + topic_len;
    if (pkt_type == CORE_MQTT_SUB_PKT_TYPE) {
        remainlen += CORE_MQTT_REQUEST_QOS_LEN;
    }
    pkt_len = CORE_MQTT_FIXED_HEADER_LEN + CORE_MQTT_REMAINLEN_MAXLEN + remainlen;

    pkt = sysdep->core_sysdep_malloc(pkt_len, MQTT_IMPL);
    if (pkt == NULL) {
        return STATE_SYS_DEPEND_MALLOC_FAILED;
    }
    memset(pkt, 0, pkt_len);

    /* Subscribe/Unsubscribe Packet Type */
    if (pkt_type == CORE_MQTT_SUB_PKT_TYPE) {
        pkt[idx++] = CORE_MQTT_SUB_PKT_TYPE | CORE_MQTT_SUB_PKT_RESERVE;
    } else if (pkt_type == CORE_MQTT_UNSUB_PKT_TYPE) {
        pkt[idx++] = CORE_MQTT_UNSUB_PKT_TYPE;
    }

    /* Remaining Length */
    _core_mqtt_remain_len_encode(remainlen, &pkt[idx], &idx);

    /* Packet Id */
    packet_id = _core_mqtt_packet_id(mqtt_client);
    pkt[idx++] = (uint8_t)((packet_id >> 8) & 0x00FF);
    pkt[idx++] = (uint8_t)((packet_id) & 0x00FF);

    /* Topic */
    pkt[idx++] = (uint8_t)((topic_len >> 8) & 0x00FF);
    pkt[idx++] = (uint8_t)((topic_len) & 0x00FF);
    memcpy(&pkt[idx], topic, topic_len);
    idx += topic_len;

    /* QOS */
    if (pkt_type == CORE_MQTT_SUB_PKT_TYPE) {
        pkt[idx++] = qos;
    }

    pkt_len = idx;

    res = _core_mqtt_write(mqtt_client, pkt, pkt_len, mqtt_client->send_timeout_ms);
    if (res < STATE_SUCCESS) {
        return res;
    }

    sysdep->core_sysdep_free(pkt);

    return packet_id;
}

static void* mqtt_init()
{
    mqtt_client_t *mqtt_client = NULL;
    sysdep = aiot_sysdep_get_portfile();
    if (sysdep == NULL) {
        return NULL;
    }

    mqtt_client = sysdep->core_sysdep_malloc(sizeof(mqtt_client_t), MQTT_IMPL);
    if (mqtt_client == NULL) {
        return NULL;
    }
    memset(mqtt_client, 0, sizeof(mqtt_client_t));
    mqtt_client->data_mutex = sysdep->core_sysdep_mutex_init();

    return mqtt_client;
}

static int32_t mqtt_connect(void *handle, conn_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    core_sysdep_socket_type_t socket_type = CORE_SYSDEP_SOCKET_TCP_CLIENT;
    uint8_t *conn_pkt = NULL;
    uint32_t conn_pkt_len = 0;
    uint8_t connack_fixed_header = 0;
    uint8_t *connack_ptr = NULL;
    uint32_t remain_len = 0;
    int32_t res = STATE_SUCCESS;
    if(handle == NULL || params == NULL) {
        return STATE_PORT_INPUT_NULL_POINTER;
    }

    mqtt_client->network_handle = sysdep->core_sysdep_network_init();
    if (mqtt_client->network_handle == NULL) {
        return STATE_SYS_DEPEND_MALLOC_FAILED;
    }

    if ((res = sysdep->core_sysdep_network_setopt(mqtt_client->network_handle, CORE_SYSDEP_NETWORK_SOCKET_TYPE,
               &socket_type)) < STATE_SUCCESS ||
        (res = sysdep->core_sysdep_network_setopt(mqtt_client->network_handle, CORE_SYSDEP_NETWORK_HOST,
                params->host)) < STATE_SUCCESS ||
        (res = sysdep->core_sysdep_network_setopt(mqtt_client->network_handle, CORE_SYSDEP_NETWORK_PORT,
                &params->port)) < STATE_SUCCESS ||
        (res = sysdep->core_sysdep_network_setopt(mqtt_client->network_handle,
                CORE_SYSDEP_NETWORK_CONNECT_TIMEOUT_MS,
                &params->connect_timeout_ms)) < STATE_SUCCESS) {
        sysdep->core_sysdep_network_deinit(&mqtt_client->network_handle);
        return res;
    }
    if(params->cred != NULL) {
        if ((res = sysdep->core_sysdep_network_setopt(mqtt_client->network_handle, CORE_SYSDEP_NETWORK_CRED,
                   params->cred)) < STATE_SUCCESS) {
            sysdep->core_sysdep_network_deinit(&mqtt_client->network_handle);
            return res;
        }
        if (params->cred->option == AIOT_SYSDEP_NETWORK_CRED_SVRCERT_PSK) {
            core_sysdep_psk_t sysdep_psk = {params->psk_id, params->psk};
            res = sysdep->core_sysdep_network_setopt(mqtt_client->network_handle, CORE_SYSDEP_NETWORK_PSK,
                    (void *)&sysdep_psk);
            if (res < STATE_SUCCESS) {
                return res;
            }
        }
    }

    if ((res = sysdep->core_sysdep_network_establish(mqtt_client->network_handle)) < STATE_SUCCESS) {
        sysdep->core_sysdep_network_deinit(&mqtt_client->network_handle);
        return res;
    }

    /* Get MQTT Connect Packet */
    res = _core_mqtt_conn_pkt(params, &conn_pkt, &conn_pkt_len);
    if (res < STATE_SUCCESS) {
        return res;
    }

    mqtt_client->send_timeout_ms = params->send_timeout_ms;
    mqtt_client->recv_timeout_ms = params->recv_timeout_ms;
    mqtt_client->event_handler = params->event_handler;
    mqtt_client->userdata = params->userdata;

    /* Send MQTT Connect Packet */
    res = _core_mqtt_write(mqtt_client, conn_pkt, conn_pkt_len, mqtt_client->send_timeout_ms);
    sysdep->core_sysdep_free(conn_pkt);
    if (res < STATE_SUCCESS) {
        if (res == STATE_SYS_DEPEND_NWK_WRITE_LESSDATA) {
            core_log1(sysdep, STATE_MQTT_LOG_CONNECT_TIMEOUT, "MQTT connect packet send timeout: %d\r\n",
                      &mqtt_client->send_timeout_ms);
        } else {
            if (mqtt_client->network_handle != NULL) {
                sysdep->core_sysdep_network_deinit(&mqtt_client->network_handle);
            }
        }
        return res;
    }

    /* Receive MQTT Connect ACK Fixed Header Byte */
    res = _core_mqtt_read(mqtt_client, &connack_fixed_header, CORE_MQTT_FIXED_HEADER_LEN, mqtt_client->recv_timeout_ms);
    if (res < STATE_SUCCESS) {
        if (res == STATE_SYS_DEPEND_NWK_READ_LESSDATA) {
            core_log1(sysdep, STATE_MQTT_LOG_CONNECT_TIMEOUT, "MQTT connack packet recv fixed header timeout: %d\r\n",
                      &mqtt_client->recv_timeout_ms);
        } else {
            if (mqtt_client->network_handle != NULL) {
                sysdep->core_sysdep_network_deinit(&mqtt_client->network_handle);
            }
        }
        return res;
    }

    if (connack_fixed_header != CORE_MQTT_CONNACK_PKT_TYPE) {
        return STATE_MQTT_CONNACK_FMT_ERROR;
    }

    /* Receive MQTT Connect ACK remain len */
    res = _core_mqtt_read_remainlen(mqtt_client, &remain_len);
    if (res < STATE_SUCCESS) {
        if (res == STATE_SYS_DEPEND_NWK_READ_LESSDATA) {
            core_log1(sysdep, STATE_MQTT_LOG_CONNECT_TIMEOUT, "MQTT connack packet recv remain len timeout: %d\r\n",
                      &mqtt_client->recv_timeout_ms);
        }
        return res;
    }

    /* Connack Format Error for Mqtt 3.1 */
    if (remain_len != 0x2) {
        return STATE_MQTT_CONNACK_FMT_ERROR;
    }

    connack_ptr = sysdep->core_sysdep_malloc(remain_len, MQTT_IMPL);
    if (NULL == connack_ptr) {
        return STATE_SYS_DEPEND_MALLOC_FAILED;
    }
    memset(connack_ptr, 0, remain_len);

    /* Receive MQTT Connect ACK Variable Header */
    res = _core_mqtt_read(mqtt_client, connack_ptr, remain_len, mqtt_client->recv_timeout_ms);

    if (res < STATE_SUCCESS) {
        if (res == STATE_SYS_DEPEND_NWK_READ_LESSDATA) {
            core_log1(sysdep, STATE_MQTT_LOG_CONNECT_TIMEOUT,
                      "MQTT connack packet variable header recv timeout: %d\r\n",
                      &mqtt_client->recv_timeout_ms);
        }
        sysdep->core_sysdep_free(connack_ptr);
        return res;
    }

    res = _core_mqtt_connack_handle(mqtt_client, connack_ptr, remain_len);
    sysdep->core_sysdep_free(connack_ptr);
    if (res < STATE_SUCCESS) {
        sysdep->core_sysdep_network_deinit(&mqtt_client->network_handle);
        return res;
    }

    core_log1(sysdep, 0,
                "mqtt connect result: %d\r\n",
                &res);
    return STATE_MQTT_CONNECT_SUCCESS;
}

static int32_t mqtt_disconnect(void *handle, disconn_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    uint8_t pingreq_pkt[2] = {CORE_MQTT_DISCONNECT_PKT_TYPE, 0x00};

    return _core_mqtt_write(mqtt_client, pingreq_pkt, 2, mqtt_client->send_timeout_ms);
}

static int32_t mqtt_sub(void *handle, sub_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }
    return _core_mqtt_subunsub(mqtt_client, params->topic, strlen(params->topic), params->qos, CORE_MQTT_SUB_PKT_TYPE);
}

static int32_t mqtt_unsub(void *handle, unsub_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }
    return _core_mqtt_subunsub(mqtt_client, params->topic, strlen(params->topic), 1, CORE_MQTT_UNSUB_PKT_TYPE);
}

static int32_t _core_mqtt_pub(void *handle, char *topic, uint8_t *payload, uint32_t payload_len, uint8_t qos, uint16_t packet_id)
{
    int32_t res = STATE_SUCCESS;
    uint8_t *pkt = NULL;
    uint32_t idx = 0, remainlen = 0, pkt_len = 0;
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;

    remainlen = strlen(topic) + payload_len + CORE_MQTT_UTF8_STR_EXTRA_LEN;
    if (qos == CORE_QOS1) {
        remainlen += CORE_MQTT_PACKETID_LEN;
    }

    pkt_len = CORE_MQTT_FIXED_HEADER_LEN + CORE_MQTT_REMAINLEN_MAXLEN + remainlen;
    pkt = sysdep->core_sysdep_malloc(pkt_len, MQTT_IMPL);
    if (pkt == NULL) {
        return STATE_SYS_DEPEND_MALLOC_FAILED;
    }
    memset(pkt, 0, pkt_len);

    /* Publish Packet Type */
    pkt[idx++] = CORE_MQTT_PUBLISH_PKT_TYPE | (qos << 1);

    /* Remaining Length */
    _core_mqtt_remain_len_encode(remainlen, &pkt[idx], &idx);

    /* Topic */
    _core_mqtt_set_utf8_encoded_str((uint8_t *)topic, strlen(topic), &pkt[idx]);
    idx += CORE_MQTT_UTF8_STR_EXTRA_LEN + strlen(topic);

    /* Packet Id For QOS 1*/
    if (qos == CORE_QOS1) {
        if(packet_id == 0) {
            packet_id = _core_mqtt_packet_id(mqtt_client);
        }
        pkt[idx++] = (uint8_t)((packet_id >> 8) & 0x00FF);
        pkt[idx++] = (uint8_t)((packet_id) & 0x00FF);
    }

    /* Payload */
    memcpy(&pkt[idx], payload, payload_len);
    idx += payload_len;

    pkt_len = idx;

    res = _core_mqtt_write(handle, pkt, pkt_len, mqtt_client->send_timeout_ms);
    sysdep->core_sysdep_free(pkt);
    if(res > 0) {
        if (qos == CORE_QOS1) {
            res = packet_id;
        } else {
            res = STATE_SUCCESS;
        }
    }

    return res;
}

static int32_t mqtt_pub(void *handle, pub_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }
    return _core_mqtt_pub(mqtt_client, params->topic, params->payload, params->payload_len, params->qos, params->packet_id);
}

static int32_t mqtt_heartbeat(void *handle)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    uint8_t pingreq_pkt[2] = { CORE_MQTT_PINGREQ_PKT_TYPE, 0x00 };
    int32_t res = STATE_SUCCESS;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }
    res = _core_mqtt_write(mqtt_client, pingreq_pkt, 2, mqtt_client->send_timeout_ms);
    if(res > 0) {
        res = STATE_SUCCESS;
    }

    return res;
}
static int32_t _core_mqtt_puback_send(mqtt_client_t *mqtt_client, uint16_t packet_id)
{
    uint8_t pkt[4] = {0};

    /* Packet Type */
    pkt[0] = CORE_MQTT_PUBACK_PKT_TYPE;

    /* Remaining Lengh */
    pkt[1] = 2;

    /* Packet Id */
    pkt[2] = (uint16_t)((packet_id >> 8) & 0x00FF);
    pkt[3] = (uint16_t)((packet_id) & 0x00FF);

    return _core_mqtt_write(mqtt_client, pkt, 4, mqtt_client->send_timeout_ms);
}

static int32_t mqtt_puback(void *handle, uint16_t packet_id)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    int32_t res = STATE_SUCCESS;
    if(mqtt_client == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }
    res = _core_mqtt_puback_send(mqtt_client, packet_id);
    if(res > 0) {
        res = STATE_SUCCESS;
    }

    return res;
}

static int32_t _core_mqtt_pingresp_handler(mqtt_client_t *mqtt_client, uint8_t *input, uint32_t len, aiot_mqtt_recv_t *packet)
{
    if (len != 0) {
        return STATE_MQTT_RECV_INVALID_PINRESP_PACKET;
    }

    memset(packet, 0, sizeof(aiot_mqtt_recv_t));
    packet->type = AIOT_MQTTRECV_HEARTBEAT_RESPONSE;
    return STATE_SUCCESS;
}

static int32_t _core_mqtt_pub_handler(mqtt_client_t *mqtt_client, uint8_t *input, uint32_t len, uint8_t qos, aiot_mqtt_recv_t *packet)
{
    uint32_t idx = 0;
    uint16_t packet_id = 0;
    uint16_t utf8_strlen = 0;
    char *topic = NULL;
    uint8_t *payload = NULL;
    uint32_t topic_len = 0, payload_len = 0;

    if (input == NULL || len == 0 || qos > CORE_QOS1) {
        return STATE_MQTT_RECV_INVALID_PUBLISH_PACKET;
    }

    /* Topic Length */
    if (len < 2) {
        return STATE_MQTT_RECV_INVALID_PUBLISH_PACKET;
    }

    memset(packet, 0, sizeof(*packet));

    utf8_strlen = input[idx++] << 8;
    utf8_strlen |= input[idx++];

    topic = (char *)&input[idx];
    topic_len = utf8_strlen;
    idx += utf8_strlen;

    /* Packet Id For QOS1 */
    if (len < idx) {
        return STATE_MQTT_RECV_INVALID_PUBLISH_PACKET;
    }
    if (qos == CORE_QOS1) {
        if (len < idx + 2) {
            return STATE_MQTT_RECV_INVALID_PUBLISH_PACKET;
        }
        packet_id = input[idx++] << 8;
        packet_id |= input[idx++];
    }

    /* Payload Len */
    if ((int64_t)len - CORE_MQTT_PUBLISH_TOPICLEN_LEN - (int64_t)topic_len < 0) {
        return STATE_MQTT_RECV_INVALID_PUBLISH_PACKET;
    }
    payload = &input[idx];
    payload_len = len - CORE_MQTT_PUBLISH_TOPICLEN_LEN - topic_len;

    if (qos == CORE_QOS1) {
        payload_len -= CORE_MQTT_PACKETID_LEN;
    }
    
    memset(packet, 0, sizeof(aiot_mqtt_recv_t));
    packet->type = AIOT_MQTTRECV_PUB;
    packet->data.pub.qos = qos;
    packet->data.pub.payload = sysdep->core_sysdep_malloc(payload_len, MQTT_IMPL);
    memcpy(packet->data.pub.payload, payload, payload_len);
    packet->data.pub.payload_len = payload_len;
    packet->data.pub.topic = sysdep->core_sysdep_malloc(topic_len + 1, MQTT_IMPL);
    memset(packet->data.pub.topic, 0, topic_len + 1);
    memcpy(packet->data.pub.topic, topic, topic_len);
    packet->data.pub.topic_len = topic_len;
    packet->data.pub.packet_id = packet_id;

    return STATE_SUCCESS;
}


static int32_t _core_mqtt_puback_handler(mqtt_client_t *mqtt_client, uint8_t *input, uint32_t len, aiot_mqtt_recv_t *packet)
{
    if (input == NULL || len != 2) {
        return STATE_MQTT_RECV_INVALID_PUBACK_PACKET;
    }

    memset(packet, 0, sizeof(aiot_mqtt_recv_t));
    packet->type = AIOT_MQTTRECV_PUB_ACK;

    /* Packet Id */
    packet->data.pub_ack.packet_id = input[0] << 8;
    packet->data.pub_ack.packet_id |= input[1];

    return STATE_SUCCESS;
}

static int32_t _core_mqtt_subunsuback_handler(mqtt_client_t *mqtt_client, uint8_t *input, uint32_t len,
        uint8_t packet_type, aiot_mqtt_recv_t *packet)
{
    uint32_t idx = 0;

    if (input == NULL || len == 0 ||
        (packet_type == CORE_MQTT_SUBACK_PKT_TYPE && len % 3 != 0) ||
        (packet_type == CORE_MQTT_UNSUBACK_PKT_TYPE && len % 2 != 0)) {
        return STATE_MQTT_MALFORMED_REMAINING_BYTES;
    }

    for (idx = 0; idx < len;) {
        memset(packet, 0, sizeof(aiot_mqtt_recv_t));
        if (packet_type == CORE_MQTT_SUBACK_PKT_TYPE) {
            packet->type = AIOT_MQTTRECV_SUB_ACK;
            packet->data.sub_ack.packet_id = input[idx] << 8;
            packet->data.sub_ack.packet_id |= input[idx + 1];
        } else {
            packet->type = AIOT_MQTTRECV_UNSUB_ACK;
            packet->data.unsub_ack.packet_id = input[idx] << 8;
            packet->data.unsub_ack.packet_id |= input[idx + 1];
        }

        if (packet_type == CORE_MQTT_SUBACK_PKT_TYPE) {
            if (input[idx + 2] == CORE_MQTT_SUBACK_RCODE_MAXQOS0 ||
                input[idx + 2] == CORE_MQTT_SUBACK_RCODE_MAXQOS1 ||
                input[idx + 2] == CORE_MQTT_SUBACK_RCODE_MAXQOS2) {
                packet->data.sub_ack.res = STATE_SUCCESS;
                packet->data.sub_ack.max_qos = input[idx + 2];
            } else if (input[idx + 2] == CORE_MQTT_SUBACK_RCODE_FAILURE) {
                packet->data.sub_ack.res = STATE_MQTT_SUBACK_RCODE_FAILURE;
            } else {
                packet->data.sub_ack.res = STATE_MQTT_SUBACK_RCODE_UNKNOWN;
            }
            idx += 3;
        } else {
            idx += 2;
        }
    }
    return STATE_SUCCESS;
}


static int32_t mqtt_recv(void *handle, aiot_mqtt_recv_t *packet)
{
    int32_t res = STATE_SUCCESS;
    uint32_t mqtt_remainlen = 0;
    uint8_t mqtt_fixed_header[CORE_MQTT_FIXED_HEADER_LEN] = {0};
    uint8_t mqtt_pkt_type = 0, mqtt_pkt_reserved = 0;
    uint8_t *remain = NULL;
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(mqtt_client == NULL){
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }
    /* Read Packet Type Of MQTT Fixed Header, Get  And Remaining Packet Length */
    res = _core_mqtt_read(mqtt_client, mqtt_fixed_header, CORE_MQTT_FIXED_HEADER_LEN, mqtt_client->recv_timeout_ms);
    if (res < STATE_SUCCESS) {
        return res;
    }

    /* Read Remaining Packet Length Of MQTT Fixed Header */
    res = _core_mqtt_read_remainlen(mqtt_client, &mqtt_remainlen);
    if (res < STATE_SUCCESS) {
        return res;
    }

    /* Read Remaining Bytes */
    res = _core_mqtt_read_remainbytes(handle, mqtt_remainlen, &remain);

    if (res < STATE_SUCCESS) {
        return res;
    }

    /* Get Packet Type */
    mqtt_pkt_type = mqtt_fixed_header[0] & 0xF0;
    mqtt_pkt_reserved = mqtt_fixed_header[0] & 0x0F;

    /* reset ping response missing times */
    switch (mqtt_pkt_type) {
        case CORE_MQTT_PINGRESP_PKT_TYPE: {
            res = _core_mqtt_pingresp_handler(mqtt_client, remain, mqtt_remainlen, packet);
        }
        break;
        case CORE_MQTT_PUBLISH_PKT_TYPE: {
            res = _core_mqtt_pub_handler(handle, remain, mqtt_remainlen, ((mqtt_pkt_reserved >> 1) & 0x03), packet);
        }
        break;
        case CORE_MQTT_PUBACK_PKT_TYPE: {
            res = _core_mqtt_puback_handler(handle, remain, mqtt_remainlen, packet);
        }
        break;
        case CORE_MQTT_SUBACK_PKT_TYPE: {
            res = _core_mqtt_subunsuback_handler(handle, remain, mqtt_remainlen, CORE_MQTT_SUBACK_PKT_TYPE, packet);
        }
        break;
        case CORE_MQTT_UNSUBACK_PKT_TYPE: {
            res = _core_mqtt_subunsuback_handler(handle, remain, mqtt_remainlen, CORE_MQTT_UNSUBACK_PKT_TYPE, packet);
        }
        break;
        case CORE_MQTT_PUBREC_PKT_TYPE:
        case CORE_MQTT_PUBREL_PKT_TYPE:
        case CORE_MQTT_PUBCOMP_PKT_TYPE: {
        }
        break;
        default: {
            res = STATE_MQTT_PACKET_TYPE_UNKNOWN;
        }
    }

    if (remain) {
        sysdep->core_sysdep_free(remain);
    }

    return res;
}

static int32_t mqtt_deinit(void **handle)
{
    mqtt_client_t *mqtt_client = NULL;
    if(handle == NULL || *handle == NULL) {
        return STATE_USER_INPUT_NULL_POINTER;
    }

    mqtt_client = *(mqtt_client_t **)handle;
    if (mqtt_client->network_handle != NULL) {
        sysdep->core_sysdep_network_deinit(&mqtt_client->network_handle);
    }
    if(mqtt_client->data_mutex != NULL) {
        sysdep->core_sysdep_mutex_deinit(&mqtt_client->data_mutex);
    }

    sysdep->core_sysdep_free(mqtt_client);
    *handle = NULL;
    return STATE_SUCCESS;
}

/* 如果编译器不支持weak属性，删除__attribute__((weak))描述 */
aiot_mqtt_port_t __attribute__((weak)) mqtt_impl = {
    .mqtt_init = mqtt_init,
    .mqtt_connect = mqtt_connect,
    .mqtt_disconnect = mqtt_disconnect,
    .mqtt_sub = mqtt_sub,
    .mqtt_unsub = mqtt_unsub,
    .mqtt_pub = mqtt_pub,
    .mqtt_heartbeat = mqtt_heartbeat,
    .mqtt_puback = mqtt_puback,
    .mqtt_recv = mqtt_recv,
    .mqtt_deinit = mqtt_deinit,
};