/**
 * @file aiot_at_api.c
 * @brief 提供了外部通信模组对接到SDK网络适配层的接口实现
 * @date 2020-01-20
 *
 * @copyright Copyright (C) 2015-2020 Alibaba Group Holding Limited
 *
 * @details
 * 1. 用户需将具体的模组设备结构化数据按照at_device_t格式传进来
 * 2. AT+MQTT功能实现
 * 3. 用户应根据应用的实际数据吞吐量合理配置ringbuf大小, ringbuf写入溢出会导致报文不完整, 设备会重新建连
 */

#include "core_stdinc.h"
#include "core_string.h"
#include "aiot_state_api.h"
#include "aiot_sysdep_api.h"
#include "aiot_at_mqtt_api.h"
#include "core_log.h"
#include <string.h>
#include <stdlib.h>
#include <stdio.h>

static aiot_sysdep_portfile_t *sysdep = NULL;

static core_at_handle_t at_handle = {
    .is_init = 0,
};

static at_rsp_result_t at_csq_handler(char *rsp)
{
    at_rsp_result_t res = AT_RSP_WAITING;
    int rssi = 0, ber = 0;
    char *line = NULL;
    line = strstr(rsp, "+CSQ");
    /* 获取信号强度，强度<5返回失败，否则返回成功 */
    if(line != NULL && sscanf(line, "+CSQ: %d,%d\r\n", &rssi, &ber)) {
        if(rssi < 5) {
            res =  AT_RSP_FAILED;
        } else {
            res =  AT_RSP_SUCCESS;
        }
    }

    return res;
}

/* 模组初始化命令集 */
static core_at_cmd_item_t at_module_init_cmd_table[] = {
    {   /* UART通道测试 */
        .cmd = "AT\r\n",
        .rsp = "OK",
    },
    {   /* 关闭回显 */
        .cmd = "ATE0\r\n",
        .rsp = "OK",
    },
    {   /* 获取模组型号 */
        .cmd = "ATI\r\n",
        .rsp = "OK",
    },
    {   /* 获取通信模块 IMEI 号 */
        .cmd = "AT+CGSN\r\n",
        .rsp = "OK",
    },
    {   /* 获取模组固件版本号 */
        .cmd = "AT+CGMR\r\n",
        .rsp = "OK",
    },
    {   /* 检查SIM卡 */
        .cmd = "AT+CPIN?\r\n",
        .rsp = "OK",
    },
    {   /* 获取SIM卡IMSI */
        .cmd = "AT+CIMI\r\n",
        .rsp = "OK",
    },
    {   /* 检查信号强度 */
        .cmd = "AT+CSQ\r\n",
        .rsp = "OK",
        .handler = at_csq_handler,
    },
};

/* 模块初始化命令表 */
static core_at_cmd_item_t at_ip_init_cmd_table[] = {
    {   /* 设置身份认证参数 */
        .cmd = "AT+QICSGP=1,1,\"UNINET\",\"\",\"\",1\r\n",
        .rsp = "OK",
    },
    {   /* 去场景激活 */
        .cmd = "AT+QIDEACT=1\r\n",
        .rsp = "OK",
    },
    {   /* 场景激活 */
        .cmd = "AT+QIACT=1\r\n",
        .rsp = "OK",
    },
    {   /* 查询场景激活 */
        .cmd = "AT+QIACT?\r\n",
        .rsp = "OK",
    },
};

/* TCP关闭连接AT命令表 */
static core_at_cmd_item_t at_ssl_cmd_table[] = {
    /* 配置ssl参数 */
    {
        .cmd = "AT+QSSLCFG=\"sslversion\",1,1\r\n",
        .rsp = "OK",
    },
    {
        .cmd = "AT+QSSLCFG=\"ciphersuite\",1,0X0035\r\n",
        .rsp = "OK",
    },
    {
        .cmd = "AT+QSSLCFG=\"cacert\",1,\"UFS:cacert.pem\"\r\n",
        .rsp = "OK",
    },
    {
        .cmd = "AT+QSSLCFG=\"seclevel\",1,1\r\n",
        .rsp = "OK",
    },

    /* 写服务器的跟证书*/
    {
        .cmd = "AT+QFOPEN=\"cacert.pem\",1,1\r\n",
        .rsp = "OK",
    },
    {   /*%d=证书的长度*/
        .fmt = "AT+QFWRITE=1,%d\r\n",
        .rsp = "CONNECT",
    },
    {   /*.fmt和.cmd都为NULL,为写证书数据*/
        .fmt = NULL,
        .cmd = NULL,
        .rsp = "+QFWRITE:",
    },
    {
        .cmd = "AT+QFCLOSE=1\r\n",
        .rsp = "OK",
    },
};

static int32_t at_add_packet(int32_t id, aiot_mqtt_recv_t *packet)
{
    int32_t res = -1;
    sysdep->core_sysdep_mutex_lock(at_handle.data_mutex);
    for(int i = 0; i < AT_MAX_MQTT_PACKET_SIZE; i++) {
        if(at_handle.packet[id][i] == NULL) {
            at_handle.packet[id][i] = packet;
            res = STATE_SUCCESS;
            break;
        }
    }
    sysdep->core_sysdep_mutex_unlock(at_handle.data_mutex);
    return res;
}

/* pub消息处理,返回已消费的消息长度 */
static int32_t at_recv_check_pub(uint8_t *data, uint32_t data_len)
{
    int32_t id = 0, msg_id = 0, pre_len = 0, i = 0;
    static aiot_mqtt_recv_t *packet = NULL;
    char topic[128];
    int32_t res = 0;
    const char *fmt = "+QMTRECV: %d,%d,\"%[^\"]\",%d,\"";
    char *dot = NULL, *start = NULL;
    static int32_t payload_len = 0, recv_len = 0;
    int32_t write_len = 0;

    /* 提取通知中的通道id，消息id，topic，payload长度 */
    if(payload_len == 0 && data_len > 10 && (start = strstr((char *)data, "+QMTRECV")) != NULL) {
        memset(topic, 0, sizeof(topic));
        res = sscanf((char *)start, fmt, &id, &msg_id, topic, &payload_len);
        if(res != 4 || id >= AIOT_AT_SOCKET_NUM) {
            return 0;
        }
        /* 计算前缀的描述信息长度，通过第四个逗号准备接收payload */
        dot = start;
        for(i = 0; i < 4; i++) {
            dot = strchr(dot, ',');
            if(dot == NULL) {
                return 0;
            }
        }
        pre_len = dot - start + 2;
        if(pre_len > data_len) {
            return 0;
        }

        /* 生成packet，添加仅列表中，供应用层调用 */
        packet = sysdep->core_sysdep_malloc(sizeof(aiot_mqtt_recv_t), "AT");
        if(packet == NULL) {
            return 0;
        }
        memset(packet, 0, sizeof(*packet));
        packet->type = AIOT_MQTTRECV_PUB;
        /* 存储消息id，仅当msg_id > 0时，QoS不为0 */
        packet->data.pub.packet_id = msg_id;
        if(packet->data.pub.packet_id > 0) {
            packet->data.pub.qos = 1;
        }
        /* 拷贝topic */
        core_strdup(sysdep, &packet->data.pub.topic, topic, "AT");
        packet->data.pub.topic_len = strlen(topic);
        packet->data.pub.payload = sysdep->core_sysdep_malloc(payload_len, "AT");
        if(packet->data.pub.payload == NULL) {
            sysdep->core_sysdep_free(packet);
            return 0;
        }
        memset(packet->data.pub.payload, 0, payload_len);
        packet->data.pub.payload_len = payload_len;
        return pre_len;

    } else if(payload_len > 0 && payload_len - recv_len > 0) {
        /* 接收处理payload */
        write_len = payload_len - recv_len > data_len ? data_len : payload_len - recv_len;
        memcpy(packet->data.pub.payload + recv_len, data, write_len);
        recv_len += write_len;
        if(recv_len >= payload_len) {
            at_add_packet(id, packet);
            packet = NULL;
            payload_len = recv_len = 0;
        }
        return write_len;
    }

    return 0;
}

/* sub_ack */
static void at_recv_sub_ack(char *line)
{
    int32_t id = 0, msg_id = 0, result = 0;
    int32_t res = 0;
    aiot_mqtt_recv_t *packet = NULL;
    const char *fmt = "+QMTSUB: %d,%d,%d\r\n";
    /* 提取通知中的通道id，消息id，结果 */
    res = sscanf(line, fmt, &id, &msg_id, &result);
    if(res != 3) {
        return;
    }

    if(id >= AIOT_AT_SOCKET_NUM) {
        return;
    }

    /* 生成packet等待应用层读取 */
    packet = sysdep->core_sysdep_malloc(sizeof(aiot_mqtt_recv_t), "AT");
    if(packet == NULL) {
        return;
    }
    memset(packet, 0, sizeof(*packet));
    packet->type = AIOT_MQTTRECV_SUB_ACK;
    packet->data.sub_ack.packet_id = msg_id;
    packet->data.sub_ack.res = result;

    at_add_packet(id, packet);
}

/* unsub_ack */
static void at_recv_unsub_ack(char *line)
{
    int32_t id = 0, msg_id = 0, result = 0;
    int32_t res = 0;
    aiot_mqtt_recv_t *packet = NULL;
    const char *fmt = "+QMTUNS: %d,%d,%d\r\n";
    res = sscanf(line, fmt, &id, &msg_id, &result);
    if(res != 3) {
        return;
    }

    if(id >= AIOT_AT_SOCKET_NUM) {
        return;
    }

    packet = sysdep->core_sysdep_malloc(sizeof(aiot_mqtt_recv_t), "AT");
    if(packet == NULL) {
        return;
    }
    memset(packet, 0, sizeof(*packet));
    packet->type = AIOT_MQTTRECV_UNSUB_ACK;
    packet->data.unsub_ack.packet_id = msg_id;
    at_add_packet(id, packet);
}

/* pub_ack */
static void at_recv_pub_ack(char *line)
{
    int32_t id = 0, msg_id = 0, result = 0;
    int32_t res = 0;
    aiot_mqtt_recv_t *packet = NULL;
    const char *fmt = "+QMTPUBEX: %d,%d,%d\r\n";
    res = sscanf(line, fmt, &id, &msg_id, &result);
    if(res != 3) {
        return;
    }

    /* msg_id为0,表示QoS0消息,不需要返回 */
    if(id >= AIOT_AT_SOCKET_NUM || msg_id == 0) {
        return;
    }
    packet = sysdep->core_sysdep_malloc(sizeof(aiot_mqtt_recv_t), "AT");
    if(packet == NULL) {
        return;
    }
    memset(packet, 0, sizeof(*packet));
    packet->type = AIOT_MQTTRECV_PUB_ACK;
    packet->data.pub_ack.packet_id = msg_id;
    at_add_packet(id, packet);
}

static core_at_urc_item_t at_urc_table[] = {
    {
        .prefix = "+QMTSUB:",
        .handle = at_recv_sub_ack,
    },
    {
        .prefix = "+QMTUNS:",
        .handle = at_recv_unsub_ack,
    },
    {
        .prefix = "+QMTPUBEX:",
        .handle = at_recv_pub_ack,
    },
};


static at_device_t ec200_at_cmd_ssl = {
    .ip_init_cmd = at_ip_init_cmd_table,
    .ip_init_cmd_size = sizeof(at_ip_init_cmd_table) / sizeof(core_at_cmd_item_t),

    .ssl_cmd = at_ssl_cmd_table,
    .ssl_cmd_size = sizeof(at_ssl_cmd_table) / sizeof(core_at_cmd_item_t),

    .error_prefix = "ERROR",

    .urc_register = at_urc_table,
    .urc_register_size = sizeof(at_urc_table) / sizeof(core_at_urc_item_t),
};

/*** ringbuf start ***/
int32_t core_ringbuf_init(core_ringbuf_t *rbuf, uint32_t size)
{
    if (rbuf->buf) {
        return STATE_AT_ALREADY_INITED;
    }
    memset(rbuf, 0, sizeof(core_ringbuf_t));

    rbuf->buf = sysdep->core_sysdep_malloc(size, "AT");
    if (NULL == rbuf->buf) {
        return STATE_SYS_DEPEND_MALLOC_FAILED;
    }

    rbuf->head = rbuf->buf;
    rbuf->tail = rbuf->buf;
    rbuf->end = rbuf->buf + size;
    rbuf->size = size;

    return STATE_SUCCESS;
}
void core_ringbuf_reset(core_ringbuf_t *rbuf)
{
    rbuf->tail = rbuf->buf;
    rbuf->head = rbuf->buf;
}

int32_t core_ringbuf_get_occupy(core_ringbuf_t *rbuf)
{
    uint32_t used = 0;
    if(rbuf->buf == NULL) {
        return 0;
    }

    if (rbuf->tail >= rbuf->head) {
        used = rbuf->tail - rbuf->head;
    } else {
        used = rbuf->tail - rbuf->buf + rbuf->end - rbuf->head;
    }

    return used;
}

int32_t core_ringbuf_write(core_ringbuf_t *rbuf, const uint8_t *data, uint32_t len)
{
    if (len > (rbuf->size - core_ringbuf_get_occupy(rbuf))) {
        return STATE_AT_RINGBUF_OVERFLOW;
    }
    if (rbuf->tail + len >= rbuf->end) {
        uint32_t remain_len = rbuf->end - rbuf->tail;

        memcpy(rbuf->tail, data, remain_len);
        memcpy(rbuf->buf, data + remain_len, len - remain_len);
        rbuf->tail = rbuf->buf + len - remain_len;
    } else {
        memcpy(rbuf->tail, data, len);
        rbuf->tail += len;
    }

    return len;
}

int32_t core_ringbuf_read(core_ringbuf_t *rbuf, uint8_t *data, uint32_t len)
{
    int32_t used = core_ringbuf_get_occupy(rbuf);

    if (len > used) {
        return 0;
    }

    if (rbuf->head + len >= rbuf->end) {
        uint32_t remain_len = rbuf->end - rbuf->head;

        memcpy(data, rbuf->head, remain_len);
        memcpy(data + remain_len, rbuf->buf, len - remain_len);
        rbuf->head = rbuf->buf + len - remain_len;
    } else {
        memcpy(data, rbuf->head, len);
        rbuf->head += len;
    }

    return len;
}

void core_ringbuf_deinit(core_ringbuf_t *rbuf)
{
    if (NULL == rbuf) {
        return;
    }

    if (rbuf->buf) {
        sysdep->core_sysdep_free(rbuf->buf);
    }

    memset(rbuf, 0, sizeof(core_ringbuf_t));
}
/*** ringbuf end ***/

static int32_t core_at_uart_tx(const uint8_t *p_data, uint16_t len, uint32_t timeout)
{
    int32_t res = STATE_SUCCESS;

    if (at_handle.uart_tx_func == NULL) {
        return STATE_AT_UART_TX_FUNC_NULL;
    }

    res = at_handle.uart_tx_func(p_data, len, timeout);

    return res;
}

static at_rsp_result_t core_at_rsp_process_default(const core_at_cmd_item_t *cmd, char *rsp)
{
    at_rsp_result_t res = AT_RSP_WAITING;
    if(cmd == NULL || rsp == NULL) {
        return AT_RSP_FAILED;
    }

    /* 出现期望的数据或者是error时，退出*/
    if (strstr(rsp, cmd->rsp) != NULL) {
        res = AT_RSP_SUCCESS;
    } else if (strstr(rsp, at_handle.device->error_prefix) != NULL) {
        res = AT_RSP_FAILED;
    }

    return res;
}

static int32_t core_at_wait_resp(const core_at_cmd_item_t *cmd)
{
    int32_t res = STATE_AT_GET_RSP_FAILED;
    at_rsp_result_t rsp_result = AT_RSP_SUCCESS;
    char rsp[AIOT_AT_RSP_LEN_MAXIMUM] = {0};
    uint64_t timeout_start = sysdep->core_sysdep_time();
    uint32_t timeout_ms = 0;
    uint32_t used = 0, total_len = 0;

    if(cmd->timeout_ms == 0) {
        timeout_ms = AIOT_AT_RX_TIMEOUT_DEFAULT;
    } else {
        timeout_ms = cmd->timeout_ms;
    }
    memset(rsp, 0, sizeof(rsp));
    do {
        used = core_ringbuf_get_occupy(&at_handle.rsp_rb);
        if(used <= 0) {
            sysdep->core_sysdep_sleep(AIOT_AT_RINGBUF_RETRY_INTERVAL);
            continue;
        }

        total_len += core_ringbuf_read(&at_handle.rsp_rb, (uint8_t *)(rsp + total_len), used);
        /* call user handle */
        if (cmd->handler) {
            rsp_result = cmd->handler(rsp);
            if(rsp_result == AT_RSP_SUCCESS) {
                res = STATE_SUCCESS;
                break;
            } else if(rsp_result == AT_RSP_FAILED) {
                res = STATE_AT_GET_RSP_FAILED;
                break;
            }
        }

        /* call default handle*/
        rsp_result = core_at_rsp_process_default(cmd, rsp);
        if(rsp_result == AT_RSP_SUCCESS) {
            res = STATE_SUCCESS;
            break;
        } else if(rsp_result == AT_RSP_FAILED) {
            res = STATE_AT_GET_RSP_FAILED;
            break;
        }

    } while ((sysdep->core_sysdep_time() - timeout_start) < timeout_ms);

    return res;
}

static int32_t core_at_commands_send_sync(const core_at_cmd_item_t *cmd_list, uint16_t cmd_num)
{
    uint16_t i = 0;
    int32_t res = STATE_SUCCESS;
    uint16_t retry_cnt = AIOT_AT_CMD_RETRY_TIME;

    if (NULL == cmd_list || 0 == cmd_num) {
        return STATE_USER_INPUT_NULL_POINTER;
    }
    sysdep->core_sysdep_mutex_lock(at_handle.cmd_mutex);
    core_ringbuf_reset(&at_handle.rsp_rb);
    for (; i < cmd_num; i++) {
        /* 发送AT命令 */
        /*
        core_log_hexdump(-1, 1, (unsigned char *)cmd_list[i].cmd, cmd_list[i].cmd_len);
        */
        at_handle.cmd_content = &cmd_list[i];
        res = core_at_uart_tx((uint8_t *)cmd_list[i].cmd, cmd_list[i].cmd_len, at_handle.tx_timeout);
        if (res != cmd_list[i].cmd_len) {
            res = STATE_AT_UART_TX_FAILED;
            break;
        }
        res = core_at_wait_resp(at_handle.cmd_content);
        if (res < 0) {
            if (--retry_cnt > 0) {
                i--;
                continue;
            } else {
                break;
            }
        }

        retry_cnt = AIOT_AT_CMD_RETRY_TIME;
        at_handle.cmd_content = NULL;
        res = STATE_SUCCESS;
    }
    sysdep->core_sysdep_mutex_unlock(at_handle.cmd_mutex);
    return res;
}

/**
 * AT moduel API start
 */
int32_t aiot_at_init(void)
{
    int32_t res = STATE_SUCCESS;

    if (at_handle.is_init != 0) {
        return STATE_AT_ALREADY_INITED;
    }

    sysdep = aiot_sysdep_get_portfile();
    memset(&at_handle, 0, sizeof(core_at_handle_t));
    at_handle.tx_timeout = AIOT_AT_TX_TIMEOUT_DEFAULT;
    at_handle.cmd_mutex = sysdep->core_sysdep_mutex_init();
    at_handle.send_mutex = sysdep->core_sysdep_mutex_init();
    at_handle.data_mutex = sysdep->core_sysdep_mutex_init();

    if ((res = core_ringbuf_init(&at_handle.rsp_rb, AIOT_AT_DATA_RB_SIZE_DEFAULT)) < STATE_SUCCESS) {
        return res;
    }

    at_handle.is_init = 1;
    return res;
}

static int32_t core_at_set_device(at_device_t *device)
{
    int32_t res = STATE_SUCCESS;
    int32_t i = 0;

    at_handle.device = device;
    /* 未定义模组初始化命令，使用默认的 */
    if(device->module_init_cmd == NULL) {
        at_handle.device->module_init_cmd = at_module_init_cmd_table;
        at_handle.device->module_init_cmd_size = sizeof(at_module_init_cmd_table) / sizeof(core_at_cmd_item_t);
    }

    for(i = 0; i < at_handle.device->module_init_cmd_size; i++) {
        if(at_handle.device->module_init_cmd[i].cmd != NULL) {
            at_handle.device->module_init_cmd[i].cmd_len = strlen(at_handle.device->module_init_cmd[i].cmd);
        }
    }
    for(i = 0; i < at_handle.device->ip_init_cmd_size; i++) {
        if(at_handle.device->ip_init_cmd[i].cmd != NULL) {
            at_handle.device->ip_init_cmd[i].cmd_len = strlen(at_handle.device->ip_init_cmd[i].cmd);
        }
    }

    for(i = 0; i < at_handle.device->ssl_cmd_size; i++) {
        if(at_handle.device->ssl_cmd[i].cmd != NULL) {
            at_handle.device->ssl_cmd[i].cmd_len = strlen(at_handle.device->ssl_cmd[i].cmd);
        }
    }

    return res;
}

int32_t aiot_at_setopt(aiot_at_option_t opt, void *data)
{
    if (at_handle.is_init != 1) {
        return STATE_AT_NOT_INITED;
    }
    if (NULL == data) {
        return STATE_USER_INPUT_NULL_POINTER;
    }
    if (opt >= AIOT_ATOPT_MAX) {
        return STATE_USER_INPUT_OUT_RANGE;
    }

    switch (opt) {
    case AIOT_ATOPT_UART_TX_FUNC: {
        at_handle.uart_tx_func = (aiot_at_uart_tx_func_t)data;
    }
    break;
    case AIOT_ATOPT_USER_DATA: {
        at_handle.user_data = data;
    }
    break;
    case AIOT_ATOPT_DEVICE: {
        core_at_set_device((at_device_t *)data);
    }
    break;
    default:
        break;
    }

    return STATE_SUCCESS;
}

int32_t aiot_at_bootstrap(void)
{
    int32_t res = STATE_SUCCESS;
    if (at_handle.is_init != 1) {
        return STATE_AT_NOT_INITED;
    }

    core_at_set_device(&ec200_at_cmd_ssl);
    res = core_at_commands_send_sync(at_handle.device->module_init_cmd, at_handle.device->module_init_cmd_size);
    if(STATE_SUCCESS != res) {
        return res;
    }

    res = core_at_commands_send_sync(at_handle.device->ip_init_cmd, at_handle.device->ip_init_cmd_size);
    if(STATE_SUCCESS != res) {
        return res;
    }

    return res;
}

int32_t aiot_at_nwk_open(uint8_t *socket_id)
{
    uint8_t i = 0;

    if (at_handle.is_init != 1) {
        return STATE_AT_NOT_INITED;
    }

    for (i = 0; i < sizeof(at_handle.fd) / sizeof(at_handle.fd[0]); i++) {
        if (at_handle.fd[i].link_status == CORE_AT_LINK_DISABLE) {
            *socket_id = i;
            at_handle.fd[i].link_status = CORE_AT_LINK_DISCONN;
            return STATE_SUCCESS;
        }
    }

    return STATE_AT_NO_AVAILABLE_LINK;
}

int32_t aiot_at_set_ssl(uint8_t socket_id, const char *ca_cert)
{
    int32_t res = STATE_SUCCESS, i = 0, data_index = 0;
    char tmp[AIOT_AT_CMD_LEN_MAXIMUM] = {0};


    for(i = 0; i < at_handle.device->ssl_cmd_size; i++) {
        if(at_handle.device->ssl_cmd[i].fmt != NULL) {
            if(strstr(at_handle.device->ssl_cmd[i].fmt, "%d")) {
                memset(tmp, 0, sizeof(tmp));
                snprintf(tmp, sizeof(tmp),at_handle.device->ssl_cmd[i].fmt, strlen(ca_cert));
                at_handle.device->ssl_cmd[i].cmd = tmp;
                at_handle.device->ssl_cmd[i].cmd_len = strlen(tmp);
            }
        } else if(at_handle.device->ssl_cmd[i].fmt == NULL && at_handle.device->ssl_cmd[i].cmd == NULL) {
            at_handle.device->ssl_cmd[i].cmd = (char *)ca_cert;
            at_handle.device->ssl_cmd[i].cmd_len = strlen(ca_cert);
            data_index = i;
        }
    }

    res = core_at_commands_send_sync(at_handle.device->ssl_cmd, at_handle.device->ssl_cmd_size);
    if (res == STATE_SUCCESS) {
        at_handle.fd[socket_id].link_status = CORE_AT_LINK_CONN;
    }
    if(data_index != 0) {
        at_handle.device->ssl_cmd[data_index].cmd = NULL;
    }

    return res;
}

static int32_t core_at_process_line(char *line, uint32_t len)
{
    if(at_handle.cmd_content) {
        core_ringbuf_write(&at_handle.rsp_rb, (uint8_t *)line, len);
    }

    for(int i = 0 ; i < at_handle.device->urc_register_size; i++) {
        if(strstr(line, at_handle.device->urc_register[i].prefix)) {
            at_handle.device->urc_register[i].handle(line);
            break;
        }
    }

    return STATE_SUCCESS;
}

static int32_t core_at_hal_process(uint8_t *data, uint32_t size)
{
    /*返回消费的数据长度*/
    int32_t res = size;
    char *postfix = NULL;
    char bak;
    int32_t line_len = 0, remain_len = 0, cost_len = 0;

    if(at_handle.is_init != 1) {
        return res;
    }

    /*更新接收缓存*/
    if (size > AIOT_AT_RSP_LEN_MAXIMUM - at_handle.rsp_buf_offset) {
        size = AIOT_AT_RSP_LEN_MAXIMUM - at_handle.rsp_buf_offset;
    }

    memcpy(&at_handle.rsp_buf[at_handle.rsp_buf_offset], data, size);
    at_handle.rsp_buf_offset += size;
    do {
        cost_len = at_recv_check_pub((uint8_t *)at_handle.rsp_buf, at_handle.rsp_buf_offset);
        \
        if(cost_len > 0) {
            remain_len = at_handle.rsp_buf_offset - cost_len;
            /*计算消费的数据长度*/
            if(remain_len > 0) {
                memmove(at_handle.rsp_buf, at_handle.rsp_buf + cost_len, remain_len);
            }
            memset(at_handle.rsp_buf + remain_len, 0, cost_len);
            at_handle.rsp_buf_offset -= cost_len;
        }
    } while(cost_len > 0 && remain_len > 0);

    /*判断是否为一行数据*/
    do {
        postfix = NULL;

        if((postfix = strstr(at_handle.rsp_buf, "\r\n")) != NULL) {
            line_len = postfix - at_handle.rsp_buf + 2;
        } else if((postfix = strstr(at_handle.rsp_buf, ">")) != NULL) {
            line_len = 1;
        }

        if(postfix != NULL) {
            bak = at_handle.rsp_buf[line_len];
            at_handle.rsp_buf[line_len] = 0;
            core_at_process_line(at_handle.rsp_buf, line_len);
            at_handle.rsp_buf[line_len] = bak;
            remain_len = at_handle.rsp_buf_offset - line_len;
            /*计算消费的数据长度*/
            if(remain_len > 0) {
                memmove(at_handle.rsp_buf, at_handle.rsp_buf + line_len, remain_len);
            }
            memset(at_handle.rsp_buf + remain_len, 0, line_len);
            at_handle.rsp_buf_offset -= line_len;
        }
    } while(postfix != NULL);

    return size;
}

int32_t aiot_at_hal_recv_handle(uint8_t *data, uint32_t size)
{
    int32_t recv_size = size;
    uint8_t *pData = data;
    uint32_t used = 0;
    if (NULL == data || 0 == size) {
        return 0;
    }

    while (recv_size > 0) {
        used = core_at_hal_process(pData, recv_size);
        pData += used;
        recv_size -= used;
    }
    return size;
}
int32_t aiot_at_deinit(void)
{
    if (at_handle.is_init == 0) {
        return STATE_SUCCESS;
    }

    sysdep->core_sysdep_mutex_deinit(&at_handle.cmd_mutex);
    sysdep->core_sysdep_mutex_deinit(&at_handle.send_mutex);
    sysdep->core_sysdep_mutex_deinit(&at_handle.data_mutex);
    core_ringbuf_deinit(&at_handle.rsp_rb);
    memset(&at_handle, 0, sizeof(core_at_handle_t));

    return STATE_SUCCESS;
}

/*模组的状态变化*/
int32_t core_at_notify_ip_status(core_ip_status_t status)
{
    int8_t i = 0;
    if(IP_SUCCESS != status) {
        for (i = 0; i < sizeof(at_handle.fd) / sizeof(at_handle.fd[0]); i++) {
            at_handle.fd[i].link_status = CORE_AT_LINK_DISCONN;
        }
    }

    return STATE_SUCCESS;
}

/*socket的状态变化*/
int32_t core_at_socket_status(uint32_t id, core_at_link_status_t status)
{
    at_handle.fd[id - AT_SOCKET_ID_START].link_status = status;
    return STATE_SUCCESS;
}

int32_t aiot_at_mqtt_connect(int32_t id, conn_para_t *params)
{
    core_at_cmd_item_t cmd;
    int32_t res = STATE_SUCCESS;
    char buf[512];

    memset(&cmd, 0, sizeof(cmd));
    cmd.cmd = buf;
    cmd.rsp = "OK";
    /* 配置连接参数, 接收模式：主动上报数据 */
    memset(buf, 0, sizeof(buf));
    cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTCFG=\"recv/mode\",%d,0,1\r\n", id);
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    /* 配置连接参数, 数据格式：发送用string，接收用二进制 */
    memset(buf, 0, sizeof(buf));
    cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTCFG=\"dataformat\",%d,0,0\r\n", id);
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    /* 配置TLS */
    memset(buf, 0, sizeof(buf));
    if(params->cred->option == AIOT_SYSDEP_NETWORK_CRED_SVRCERT_CA) {
        if(at_handle.device->ssl_cmd != NULL) {
            aiot_at_set_ssl(id + AT_SOCKET_ID_START, params->cred->x509_server_cert);
        }
        cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTCFG=\"SSL\",%d,1,1\r\n", id);
    } else {
        cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTCFG=\"SSL\",%d,0\r\n", id);
    }
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    /* 建立连接：只会建立TCP/TLS连接 */
    memset(buf, 0, sizeof(buf));
    cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTOPEN=%d,\"%s\",%d\r\n", id, params->host, params->port);
    cmd.rsp = "+QMTOPEN:";
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    /* 建立连接：建立MQTT连接 */
    memset(buf, 0, sizeof(buf));
    cmd.rsp = "+QMTCONN:";
    cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTCONN=%d,\"%s\",\"%s\",\"%s\"\r\n", id, params->clientid, params->username, params->password);
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    return 0;
}
int32_t aiot_at_mqtt_sub(int32_t id, sub_para_t *params)
{
    core_at_cmd_item_t cmd;
    int32_t res = STATE_SUCCESS;
    char buf[AIOT_AT_CMD_LEN_MAXIMUM];
    memset(&cmd, 0, sizeof(cmd));
    /* sub,等待OK返回 */
    memset(buf, 0, sizeof(buf));
    cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTSUB=%d,%d,\"%s\",%d\r\n", id, 1, params->topic, params->qos);
    cmd.cmd = buf;
    cmd.rsp = "OK";
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    return STATE_SUCCESS;
}
int32_t aiot_at_mqtt_unsub(int32_t id, unsub_para_t *params)
{
    core_at_cmd_item_t cmd;
    int32_t res = STATE_SUCCESS;
    char buf[AIOT_AT_CMD_LEN_MAXIMUM];

    memset(&cmd, 0, sizeof(cmd));
    /* sub,等待OK返回 */
    memset(buf, 0, sizeof(buf));
    cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTUNS=%d,%d,\"%s\"\r\n", id, 100, params->topic);
    cmd.cmd = buf;
    cmd.rsp = "OK";
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    return STATE_SUCCESS;
}
int32_t aiot_at_mqtt_pub(int32_t id, pub_para_t *params)
{
    core_at_cmd_item_t cmd[2];
    char buf[AIOT_AT_CMD_LEN_MAXIMUM];
    int32_t res = STATE_SUCCESS;

    memset(cmd, 0, sizeof(cmd));
    /* pub，等待ok返回 */
    memset(buf, 0, sizeof(buf));
    cmd[0].cmd_len = snprintf(buf, sizeof(buf), "AT+QMTPUBEX=%d,%d,0,0,\"%s\",%d\r\n", id, 0, params->topic, params->payload_len);
    cmd[0].cmd = buf;
    cmd[0].rsp = ">";

    cmd[1].cmd = (char *)params->payload;
    cmd[1].cmd_len = params->payload_len;
    cmd[1].rsp = "OK";
    res = core_at_commands_send_sync(cmd, 2);
    return res;
}

int32_t aiot_at_mqtt_disconnect(int32_t id)
{
    core_at_cmd_item_t cmd;
    int32_t res = STATE_SUCCESS;
    char buf[AIOT_AT_CMD_LEN_MAXIMUM];

    memset(&cmd, 0, sizeof(cmd));
    /* disconnect,等待OK返回 */
    memset(buf, 0, sizeof(buf));
    cmd.cmd_len = snprintf(buf, sizeof(buf), "AT+QMTDISC=%d\r\n", id);
    cmd.cmd = buf;
    cmd.rsp = "OK";
    res = core_at_commands_send_sync(&cmd, 1);
    if(res != STATE_SUCCESS) {
        return res;
    }

    return STATE_SUCCESS;
}

int32_t aiot_at_mqtt_get_packet(int32_t id, aiot_mqtt_recv_t *packet)
{
    int32_t i = 0, res = STATE_SYS_DEPEND_NWK_READ_LESSDATA;
    /* 返回packet，包含sub_ack、unsub_ack、... */
    sysdep->core_sysdep_mutex_lock(at_handle.data_mutex);
    for(i = 0; i < AT_MAX_MQTT_PACKET_SIZE; i++) {
        if(at_handle.packet[id][i] != NULL) {
            *packet = *(at_handle.packet[id][i]);
            sysdep->core_sysdep_free(at_handle.packet[id][i]);
            at_handle.packet[id][i] = NULL;
            res = STATE_SUCCESS;
            break;
        }
    }
    sysdep->core_sysdep_mutex_unlock(at_handle.data_mutex);
    return res;
}


