
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "mqtt_port.h"
#include "aiot_state_api.h"
#include "core_log.h"
#include "aiot_sysdep_api.h"
#include "aiot_at_mqtt_api.h"

#define MQTT_IMPL "MQTT"

static aiot_sysdep_portfile_t *sysdep = NULL;

typedef struct {
    int32_t state;
    uint8_t mqtt_fd;
    uint32_t send_timeout_ms;
    uint32_t recv_timeout_ms;
    aiot_mqtt_recv_handler_t recv_handler;
    aiot_mqtt_event_handler_t event_handler;
    void *userdata;
    uint16_t packet_id;
    void *data_mutex;
    void *packet_mutex;
    uint8_t heart;
} mqtt_client_t;


static void* mqtt_init()
{
    mqtt_client_t *mqtt_client = NULL;
    sysdep = aiot_sysdep_get_portfile();
    if (sysdep == NULL) {
        return NULL;
    }

    mqtt_client = sysdep->core_sysdep_malloc(sizeof(mqtt_client_t), MQTT_IMPL);
    if (mqtt_client == NULL) {
        return NULL;
    }
    memset(mqtt_client, 0, sizeof(mqtt_client_t));
    mqtt_client->data_mutex = sysdep->core_sysdep_mutex_init();
    mqtt_client->packet_mutex = sysdep->core_sysdep_mutex_init();
    return mqtt_client;
}


static int32_t mqtt_connect(void *handle, conn_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    int32_t res = STATE_SUCCESS;
    if(handle == NULL || params == NULL) {
        return STATE_PORT_INPUT_NULL_POINTER;
    }

    res = aiot_at_nwk_open(&mqtt_client->mqtt_fd);
    if(res != STATE_SUCCESS) {
        return -1;
    }
    res = aiot_at_mqtt_connect(mqtt_client->mqtt_fd, params);
    if(res < 0) {
        return res;
    }

    return STATE_MQTT_CONNECT_SUCCESS;
}

static int32_t mqtt_disconnect(void *handle, disconn_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;

    return aiot_at_mqtt_disconnect(mqtt_client->mqtt_fd);
}

static int32_t mqtt_sub(void *handle, sub_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }

    return aiot_at_mqtt_sub(mqtt_client->mqtt_fd, params);
}

static int32_t mqtt_unsub(void *handle, unsub_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }

    return aiot_at_mqtt_unsub(mqtt_client->mqtt_fd, params);
}

static int32_t mqtt_pub(void *handle, pub_para_t *params)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }

    return aiot_at_mqtt_pub(mqtt_client->mqtt_fd, params);
}

static int32_t mqtt_heartbeat(void *handle)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    int32_t res = STATE_SUCCESS;
    if(handle == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }
    mqtt_client->heart++;
    return res;
}


static int32_t mqtt_puback(void *handle, uint16_t packet_id)
{
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    if(mqtt_client == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }

    /* 不需要实现，paho在收到QoS1消息会默认回复 */
    return STATE_SUCCESS;
}

static int32_t mqtt_recv(void *handle, aiot_mqtt_recv_t *packet)
{
    int32_t res = STATE_SYS_DEPEND_NWK_READ_LESSDATA;
    mqtt_client_t *mqtt_client = (mqtt_client_t *)handle;
    uint64_t start_time = 0;
    if(mqtt_client == NULL) {
        return STATE_SYS_DEPEND_NWK_CLOSED;
    }

    if(mqtt_client->heart > 0) {
        mqtt_client->heart--;
        memset(packet, 0, sizeof(aiot_mqtt_recv_t));
        packet->type = AIOT_MQTTRECV_HEARTBEAT_RESPONSE;
        return STATE_SUCCESS;
    }

    start_time = sysdep->core_sysdep_time();
    do {
        res = aiot_at_mqtt_get_packet(mqtt_client->mqtt_fd, packet);
        if(res == STATE_SUCCESS) {
            break;
        } else {
            sysdep->core_sysdep_sleep(50);
            continue;
        }
    } while(sysdep->core_sysdep_time() - start_time < mqtt_client->recv_timeout_ms);
    return res;
}

static int32_t mqtt_deinit(void **handle)
{
    mqtt_client_t *mqtt_client = NULL;
    if(handle == NULL || *handle == NULL) {
        return STATE_USER_INPUT_NULL_POINTER;
    }

    mqtt_client = *(mqtt_client_t **)handle;
    if(mqtt_client->data_mutex != NULL) {
        sysdep->core_sysdep_mutex_deinit(&mqtt_client->data_mutex);
    }
    if(mqtt_client->packet_mutex != NULL) {
        sysdep->core_sysdep_mutex_deinit(&mqtt_client->packet_mutex);
    }

    sysdep->core_sysdep_free(mqtt_client);
    *handle = NULL;
    return STATE_SUCCESS;
}

/* 如果编译器不支持weak属性，删除__attribute__((weak))描述 */
aiot_mqtt_port_t mqtt_impl = {
    .mqtt_init = mqtt_init,
    .mqtt_connect = mqtt_connect,
    .mqtt_disconnect = mqtt_disconnect,
    .mqtt_sub = mqtt_sub,
    .mqtt_unsub = mqtt_unsub,
    .mqtt_pub = mqtt_pub,
    .mqtt_heartbeat = mqtt_heartbeat,
    .mqtt_puback = mqtt_puback,
    .mqtt_recv = mqtt_recv,
    .mqtt_deinit = mqtt_deinit,
};
