#include "main.h"
#include "cmsis_os.h"
#include <stdio.h>
#include "aiot_at_mqtt_api.h"
#include "os_net_interface.h"
#include <string.h>
#include "semphr.h"

/*huart1 for at module*/
extern UART_HandleTypeDef huart1;
/*huart3 for log printf*/
extern UART_HandleTypeDef huart3;
extern DMA_HandleTypeDef hdma_usart1_rx;
extern DMA_HandleTypeDef hdma_usart1_tx;

/* uart recv process */
static osThreadId g_uart_thread;
static uint8_t g_uart_thread_running = 0;
SemaphoreHandle_t uart_semphr = NULL;

extern aiot_os_al_t g_aiot_freertos_api;


/* ringbuf use for dma recv */
#define RING_BUFFER_SIZE            (1024)
typedef struct {
    uint8_t  data[RING_BUFFER_SIZE];
    uint16_t tail;
    uint16_t head;
} uart_ring_buffer_t;
int32_t isr_stat;
static uart_ring_buffer_t   uart_dma_rx_buf = {0};


/*=================uart recv start=====================================*/
#define  _uart_recv_complete_cb           HAL_UART_RxCpltCallback
#define  _uart_recv_half_complete_cb      HAL_UART_RxHalfCpltCallback
#define  _uart1_irq_cb                    USART1_IRQHandler

/*uart recv callback*/
static void _usart_recv_isr(UART_HandleTypeDef *huart)
{
    BaseType_t token = pdTRUE;
    if (huart == &huart1) {
        uart_dma_rx_buf.tail = RING_BUFFER_SIZE - hdma_usart1_rx.Instance->CNDTR;
        if(uart_semphr != NULL) {
            xSemaphoreGiveFromISR(uart_semphr, &token);
        }
    }
}

/*uart1 rx dma full complete interrupt callback*/
void _uart_recv_complete_cb(UART_HandleTypeDef *huart)
{
    _usart_recv_isr(huart);
}

/*uart1 rx dma half complete interrupt callback*/
void _uart_recv_half_complete_cb(UART_HandleTypeDef *huart)
{
    _usart_recv_isr(huart);
}
/*uart1 interrupt*/
void _uart1_irq_cb(void)
{
    /*check and process idle interrupt*/
    if ((__HAL_UART_GET_FLAG(&huart1, UART_FLAG_IDLE) != RESET)) {
        __HAL_UART_CLEAR_IDLEFLAG(&huart1);
        _usart_recv_isr(&huart1);
    }

    HAL_UART_IRQHandler(&huart1);
}

/*=================uart recv end=====================================*/

/*change log printf to uart3*/
int fputc(int ch, FILE *f)
{
    uint8_t temp[1] = {ch};
    HAL_UART_Transmit(&huart3, temp, 1, 2);
    return 0;
};

int g_tx_error = 0;
static HAL_StatusTypeDef K_UART_Transmit(UART_HandleTypeDef *huart, uint8_t *pData, uint16_t Size)
{
    HAL_StatusTypeDef status = HAL_OK;
    for (int32_t i = 1; i < 100000; ++i) {
        status = HAL_UART_Transmit_IT(huart, pData, Size);
        if (HAL_OK == status) {
            return status;
        } else if (HAL_BUSY == status) {
            if (HAL_UART_STATE_READY == huart->gState && HAL_LOCKED == huart->Lock && i % 50 == 0) {
                g_tx_error++;
                __HAL_UNLOCK(huart);
                continue;
            }
        } else if (HAL_ERROR == status) {
            return status;
        } else if (HAL_TIMEOUT == status) {
            continue;
        }
    }

    return status;
}

int32_t at_uart_send(uint8_t *p_data, uint16_t len, uint32_t timeout)
{
    /* 输出日志,实际业务中,应该删除日志打印,避免占用太多时间 */
    printf("\r\n[uart]>>> %.*s", len, p_data);
    if (HAL_OK == K_UART_Transmit(&huart1, (uint8_t *)p_data, len)) {
        return len;
    } else {
        return 0;
    }
}

void at_uart_recv(uint8_t *p_data, uint16_t len)
{
    /* 输出日志,实际业务中,应该删除日志打印,避免占用太多时间 */
    printf("\r\n[uart]<<< %.*s", len, p_data);
    aiot_at_hal_recv_handle(p_data, len);
}

void uart_write_thread(void const *args)
{
    uint32_t recv_len = 0;

    while (g_uart_thread_running) {
        if( xSemaphoreTake( uart_semphr, ( TickType_t ) 5 ) == pdTRUE ) {
            uint16_t tail = uart_dma_rx_buf.tail;
            if(tail == uart_dma_rx_buf.head) {
                continue;
            } else if (tail > uart_dma_rx_buf.head) {
                recv_len = tail - uart_dma_rx_buf.head;
                at_uart_recv(&uart_dma_rx_buf.data[uart_dma_rx_buf.head], recv_len);
                uart_dma_rx_buf.head = tail;
            } else {
                recv_len = RING_BUFFER_SIZE - uart_dma_rx_buf.head;
                at_uart_recv(&uart_dma_rx_buf.data[uart_dma_rx_buf.head], recv_len);
                recv_len = tail;
                at_uart_recv(&uart_dma_rx_buf.data[0], recv_len);
                uart_dma_rx_buf.head = tail;
            }
        }
    }
}


int32_t at_hal_init(void)
{
    printf("at_hal_init \r\n");

    /*设置设备系统接口及网络接口*/
    aiot_install_os_api(&g_aiot_freertos_api);

    uart_semphr = xSemaphoreCreateBinary();
    if(uart_semphr == NULL) {
        printf("semphr create failed \r\n");
        return -1;
    }
    /*enabled dma cycle recv*/
    uart_dma_rx_buf.head = uart_dma_rx_buf.tail = 0;
    if (HAL_OK != HAL_UART_Receive_DMA(&huart1, uart_dma_rx_buf.data, RING_BUFFER_SIZE)) {
        Error_Handler();
    }

    /* 用于串口数据处理 */
    g_uart_thread_running = 1;
    osThreadDef(UartTask, uart_write_thread, osPriorityRealtime, 0, 512);
    g_uart_thread = osThreadCreate(osThread(UartTask), NULL);
    if (g_uart_thread < 0) {
        return -1;
    }

    /*at_module_init*/
    int res = aiot_at_init();
    if (res < 0) {
        printf("aiot_at_init failed\r\n");
        g_uart_thread_running = 0;
        return -1;
    }

    /*设置发送接口*/
    aiot_at_setopt(AIOT_ATOPT_UART_TX_FUNC, at_uart_send);

    /*初始化模组及获取到IP网络*/
    res = aiot_at_bootstrap();
    if (res < 0) {
        printf("aiot_at_bootstrap failed\r\n");
        g_uart_thread_running = 0;
        return -1;
    }

    return 0;
}

int32_t at_hal_deinit(void)
{
    g_uart_thread_running = 0;
    vSemaphoreDelete(uart_semphr);
    aiot_at_deinit();
    return 0;
}
